﻿using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

// VaM Rotator Plugin by Captain Varghoss
//
// Simple plugin to enable rotation on XYZ axis with variable speed and direction.
//
// Version 3
// Added Actions: DisableAndZeroRotation & EnableAndContinueRotation
// Thanks to SallyW for this addition. https://hub.virtamate.com/members/sally-whitemane.26180/
//
// Version 2
// Fixed bug with reverse toggle trigger not working.

namespace CaptainVarghoss {
	public class VaMRotator : MVRScript {

		private Atom _atom;
		private JSONStorableBool isActive;
		private JSONStorableBool xWorld;
		private JSONStorableBool yWorld;
		private JSONStorableBool zWorld;
		private JSONStorableBool RotatorX;
		private JSONStorableBool RotatorY;
		private JSONStorableBool RotatorZ;
		private JSONStorableBool RotatorXReverse;
		private JSONStorableBool RotatorYReverse;
		private JSONStorableBool RotatorZReverse;
		private JSONStorableFloat RotatorXSpeed;
		private JSONStorableFloat RotatorYSpeed;
		private JSONStorableFloat RotatorZSpeed;
		private JSONStorableFloat RotatorXMulti;
		private JSONStorableFloat RotatorYMulti;
		private JSONStorableFloat RotatorZMulti;
		private JSONStorableString helpText;
		private JSONStorableString stateInfos;
		private UIDynamic xSpacer;
		private UIDynamic ySpacer;
		private UIDynamic zSpacer;
		
		public override void Init() {
			try {
				// Grab Containing Atom
				_atom = containingAtom;
				
				// Configure Plugin UI
				
				// Active Toggle
				isActive = new JSONStorableBool ("Active", true);
				RegisterBool (isActive);
				CreateToggle(isActive);
				
				JSONStorableAction ToggleActive = new JSONStorableAction("Toggle VaMRotator", () =>
                {
                    isActive.val = !isActive.val;
                });
                RegisterAction(ToggleActive);

                JSONStorableAction DisableAndZeroRotation = new JSONStorableAction("DisableAndZeroRotation", () =>
                {
		            try {
						isActive.val = false;
						var atomControl = _atom.GetStorableByID("control");
						atomControl.transform.rotation = Quaternion.identity;
					}
					catch (Exception e) {
						SuperController.LogError("Exception caught: " + e);
					}
                });
                RegisterAction(DisableAndZeroRotation);

                JSONStorableAction EnableAndContinueRotation = new JSONStorableAction("EnableAndContinueRotation", () =>
                {
					isActive.val = true;
                });
                RegisterAction(EnableAndContinueRotation);
				
				// Rotation Settings
				RotatorX = new JSONStorableBool("X Axis Rotation", false, VaMRotatorRebuildUI);
				RotatorY = new JSONStorableBool("Y Axis Rotation", false, VaMRotatorRebuildUI);
				RotatorZ = new JSONStorableBool("Z Axis Rotation", false, VaMRotatorRebuildUI);
				RotatorXReverse = new JSONStorableBool("Reverse X Rotation", false);
				RotatorYReverse = new JSONStorableBool("Reverse Y Rotation", false);
				RotatorZReverse = new JSONStorableBool("Reverse Z Rotation", false);
				xWorld = new JSONStorableBool ("X Axis Use World Coordinates", true);
				yWorld = new JSONStorableBool ("Y Axis Use World Coordinates", true);
				zWorld = new JSONStorableBool ("Z Axis Use World Coordinates", true);
				RotatorXSpeed = new JSONStorableFloat("X Axis Rotation Speed", 10f, 0.01f, 50f);
				RotatorYSpeed = new JSONStorableFloat("Y Axis Rotation Speed", 10f, 0.01f, 50f);
				RotatorZSpeed = new JSONStorableFloat("Z Axis Rotation Speed", 10f, 0.01f, 50f);
				RotatorXMulti = new JSONStorableFloat("X Axis Speed Multiplier", 1f, 0.01f, 10f);
				RotatorYMulti = new JSONStorableFloat("Y Axis Speed Multiplier", 1f, 0.01f, 10f);
				RotatorZMulti = new JSONStorableFloat("Z Axis Speed Multiplier", 1f, 0.01f, 10f);

				RegisterBool(RotatorX);
				RegisterBool(RotatorY);
				RegisterBool(RotatorZ);
				RegisterBool(RotatorXReverse);
				RegisterBool(RotatorYReverse);
				RegisterBool(RotatorZReverse);

				RegisterBool (xWorld);
				RegisterBool (yWorld);
				RegisterBool (zWorld);

				RegisterFloat(RotatorXSpeed);
				RegisterFloat(RotatorYSpeed);
				RegisterFloat(RotatorZSpeed);

				RegisterFloat(RotatorXMulti);
				RegisterFloat(RotatorYMulti);
				RegisterFloat(RotatorZMulti);
				
				JSONStorableString headerBox = new JSONStorableString("Header","\n<b>VaM Rotator</b>\n");
				UIDynamic headerBoxField = CreateTextField(headerBox, false);
				headerBoxField.height = 100f;

				VaMRotatorBuildUI();

				helpText = new JSONStorableString("Helptext", "\n\nEnable XYZ rotation then adjust speed sliders.\n\nUse the <b>Reverse</b> checkbox if you want the rotation to move in the opposite direction.\n\nUnchecking the <i>Use World Coordinates</i> option will force rotation to local space.\n\nAdjust the <b>multiplier</b> sliders to multiply the rotation speed on that axis.");
				UIDynamic helpTextfield = CreateTextField(helpText, false);
				helpTextfield.height = 700f;
				
				stateInfos = new JSONStorableString("Stateinfos", "");
//				UIDynamic stateinfosTextfield = CreateTextField(stateInfos, true);
//				stateinfosTextfield.height = 380.0f;
				
				SuperController.LogMessage("VaMRotator Loaded on atom: " +_atom.name);
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
				if (isActive.val) {
					var atomControl = _atom.GetStorableByID("control");
					if (RotatorX.val) {
						atomControl.transform.Rotate ( new Vector3((RotatorXReverse.val) ? -1 : 1,0,0) * ( (RotatorXSpeed.val * RotatorXMulti.val) * Time.deltaTime ) ,(xWorld.val) ? Space.World : Space.Self);
					}
					if (RotatorY.val) {
						atomControl.transform.Rotate ( new Vector3(0,(RotatorYReverse.val) ? -1 : 1,0) * ( (RotatorYSpeed.val * RotatorYMulti.val) * Time.deltaTime ) ,(yWorld.val) ? Space.World : Space.Self);
					}
					if (RotatorZ.val) {
						atomControl.transform.Rotate ( new Vector3(0,0,(RotatorZReverse.val) ? -1 : 1) * ( (RotatorZSpeed.val * RotatorZMulti.val) * Time.deltaTime ) ,(zWorld.val) ? Space.World : Space.Self);
					}
				}
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		void VaMRotatorBuildUI() {
			CreateToggle(RotatorX, true);
			if (RotatorX.val) {
				CreateToggle(RotatorXReverse, true);
				CreateToggle(xWorld, true);
				CreateSlider(RotatorXSpeed, true);
				CreateSlider(RotatorXMulti, true);
				xSpacer = CreateSpacer(true);
			}

			CreateToggle(RotatorY, true);
			if (RotatorY.val) {
				CreateToggle(RotatorYReverse, true);
				CreateToggle(yWorld, true);
				CreateSlider(RotatorYSpeed, true);
				CreateSlider(RotatorYMulti, true);
				ySpacer = CreateSpacer(true);
			}

			CreateToggle(RotatorZ, true);
			if (RotatorZ.val) {
				CreateToggle(RotatorZReverse, true);
				CreateToggle(zWorld, true);				
				CreateSlider(RotatorZSpeed, true);
				CreateSlider(RotatorZMulti, true);
				zSpacer = CreateSpacer(true);
			}
		}
		
		void VaMRotatorDestroyUI() {
			
				RemoveToggle(RotatorX);
				RemoveToggle(RotatorXReverse );
				RemoveToggle(xWorld);
				RemoveSlider(RotatorXSpeed);
				RemoveSlider(RotatorXMulti);
				RemoveSpacer(xSpacer);

				RemoveToggle(RotatorY);
				RemoveToggle(RotatorYReverse);
				RemoveToggle(yWorld);
				RemoveSlider(RotatorYSpeed);
				RemoveSlider(RotatorYMulti);
				RemoveSpacer(ySpacer);

				RemoveToggle(RotatorZ);
				RemoveToggle(RotatorZReverse);
				RemoveToggle(zWorld);				
				RemoveSlider(RotatorZSpeed);
				RemoveSlider(RotatorZMulti);
				RemoveSpacer(zSpacer);
		}

		void VaMRotatorRebuildUI(bool arg) {
			VaMRotatorDestroyUI();
			VaMRotatorBuildUI();
		}
		
		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}
		
	}
}